=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-06-07 10:45:59 +0000
+++ bzrlib/inventory.py	2006-06-09 07:35:13 +0000
@@ -906,6 +906,37 @@
                 # if we finished all children, pop it off the stack
                 stack.pop()
 
+    def iter_entries_by_dir(self, from_dir=None):
+        """Iterate over the entries in a directory first order.
+
+        This returns all entries for a directory before returning
+        the entries for children of a directory. This is not
+        lexicographically sorted order, and is a hybrid between
+        depth-first and breadth-first.
+
+        :return: This yields (path, entry) pairs
+        """
+        if from_dir == None:
+            assert self.root
+            from_dir = self.root
+        elif isinstance(from_dir, basestring):
+            from_dir = self._byid[from_dir]
+            
+        stack = [(u'', from_dir)]
+        while stack:
+            cur_relpath, cur_dir = stack.pop()
+
+            child_dirs = []
+            for child_name, child_ie in sorted(cur_dir.children.iteritems()):
+
+                child_relpath = cur_relpath + child_name
+
+                yield child_relpath, child_ie
+
+                if child_ie.kind == 'directory':
+                    child_dirs.append((child_relpath+'/', child_ie))
+            stack.extend(reversed(child_dirs))
+
     def entries(self):
         """Return list of (path, ie) for all entries except the root.
 

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-06-08 15:08:04 +0000
+++ bzrlib/tests/test_inv.py	2006-06-09 07:35:13 +0000
@@ -94,6 +94,32 @@
             ('src/hello.c', 'hello-id'),
             ], [(path, ie.file_id) for path, ie in inv.iter_entries()])
             
+    def test_iter_entries_by_dir(self):
+        inv = Inventory()
+        
+        for args in [('src', 'directory', 'src-id'), 
+                     ('doc', 'directory', 'doc-id'), 
+                     ('src/hello.c', 'file', 'hello-id'),
+                     ('src/bye.c', 'file', 'bye-id'),
+                     ('zz', 'file', 'zz-id'),
+                     ('src/sub/', 'directory', 'sub-id'),
+                     ('src/zz.c', 'file', 'zzc-id'),
+                     ('src/sub/a', 'file', 'a-id'),
+                     ('Makefile', 'file', 'makefile-id')]:
+            inv.add_path(*args)
+
+        self.assertEqual([
+            ('Makefile', 'makefile-id'),
+            ('doc', 'doc-id'),
+            ('src', 'src-id'),
+            ('zz', 'zz-id'),
+            ('src/bye.c', 'bye-id'),
+            ('src/hello.c', 'hello-id'),
+            ('src/sub', 'sub-id'),
+            ('src/zz.c', 'zzc-id'),
+            ('src/sub/a', 'a-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir()])
+            
     def test_version(self):
         """Inventory remembers the text's version."""
         inv = Inventory()

